@@ -523,28 +523,6 @@ class Date
     define_method(n.downcase + '?') { wday == i }
   end
 
-  # The relationship operator for Date.
-  #
-  # Compares dates by Julian Day Number.  When comparing
-  # two DateTime instances, or a DateTime with a Date,
-  # the instances will be regarded as equivalent if they
-  # fall on the same date in local time.
-  def === (other)
-    case other
-    when Numeric
-      jd == other
-    when Date
-      jd == other.jd
-    else
-      begin
-        l, r = other.coerce(self)
-        l === r
-      rescue NoMethodError
-        false
-      end
-    end
-  end
-
   # Step the current date forward +step+ days at a
   # time (or backward, if +step+ is negative) until
   # we reach +limit+ (inclusive), yielding the resultant
